/*!
 * @file signal_connector.h
 * @brief Definition of a SignalConnector class
*/

#ifndef _SIGNAL_CONNECTOR_H
#define _SIGNAL_CONNECTOR_H

#include "executor/executor.h"

namespace OrxGui
{
  /**
   * @brief declares a new Signal.
   * @param SignalName the Name of the Signal.
   * @param params the Parameters to be taken
   */
#define DeclareSignal(SignalName, params) \
  public: \
   void signal_ ## connect ## SignalName(const SignalConnector& connector) { \
     SignalName ## connected.push_back(connector); \
   }\
   Signal& getSignalVector_##SignalName () { return this->SignalName ## connected; }; \
  private: \
   void  SignalName params { \
     for (unsigned int i = 0; i < SignalName ## connected .size(); i++) \
       SignalName ## connected[i] ("TEST"); \
   }\
   Signal SignalName ## connected

   /**
    * @brief selects a Signal.
    * @param SignalName the Signal to be retrieved.
    */
#define SIGNAL(SignalName) \
    getSignalVector_##SignalName()

#define SLOT(Class, function) \
    createExecutor<Class>(&Class::function)

  //! A class for Conncting Signals to Objects, inside of the GUI
  class SignalConnector
  {
  public:
    SignalConnector();
    SignalConnector(BaseObject* object, const Executor* exec);
    SignalConnector(const SignalConnector& signalConnector);
    ~SignalConnector();

    SignalConnector& operator=(const SignalConnector& signalConnector);

    void operator()(const std::string& parameters) const;
    void execute(const std::string& parameters) const { (*this)(parameters); };

    bool isClean() const { return (this->object == NULL || this->exec == NULL); }


  private:
    BaseObject*          object;         //!< The object to call.
    const Executor*      exec;           //!< The Executor, that will be called, on object.
  };

  typedef std::vector<SignalConnector> Signal;
  typedef Executor*                    Slot;

}
#endif /* _SIGNAL_CONNECTOR_H */
